﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Collections.Specialized;
using System.Text.RegularExpressions;
using System.ServiceModel;

namespace DacIEServiceCli
{
    public partial class Program
    {
        /// <summary>
        /// This method processes the command line arguments and determines what action should occur.
        /// </summary>
        /// <param name="args">The list of command line arguments.</param>
        /// <returns>An action to perform or HELP if there was an error.</returns>
        internal Action ProcessCommandLine(string[] args)
        {
            Action result = Action.Invalid;
            for (int i = 0; i < args.Length; i++)
            {
                if (args[i][0] == '/' || args[i][0] == '-')
                {
                    string command = args[i].Substring(1);
                    switch (command.ToUpper())
                    {
                        case "SERVER":
                        case "S":
                            this.connectionInfo.ServerName = args[i + 1];
                            i++;
                            break;

                        case "USER":
                        case "U":
                            this.connectionInfo.UserName = args[i + 1];
                            i++;
                            break;

                        case "PASSWORD":
                        case "P":
                            this.connectionInfo.Password = args[i + 1];
                            i++;
                            break;

                        case "BLOBURL":
                        case "URL":
                            this.blobUrl = args[i + 1];
                            i++;
                            break;

                        case "BLOBACCESSKEY":
                        case "K":
                            this.blobAccessKey = args[i + 1];
                            i++;
                            break;

                        case "BLOBACCESSKEYTYPE":
                        case "ACCESSKEYTYPE":
                        case "T":
                            this.blobAccessKeyType = args[i + 1];
                            i++;
                            switch (this.blobAccessKeyType.ToUpper())
                            {
                                case "STORAGE":
                                    this.blobAccessKeyType = Program.StorageAccessKey;
                                    break;
                                case "SHARED":
                                    this.blobAccessKeyType = Program.SharedAccessKey;
                                    break;
                                default:
                                    throw new ArgumentException("Invalid argument for Azure Storage Key.  (STORAGE or SHARED)");
                            }
                            break;

                        case "EXPORT":
                        case "X":
                            result = Action.Export;
                            break;

                        case "SELECTIVEEXPORT":
                        case "SX":
                            result = Action.SelectiveExport;
                            break;

                        case "MASKEDEXPORT":
                        case "MX":
                            result = Action.MaskedExport;
                            break;

                        case "DATABASE":
                        case "D":
                            this.connectionInfo.DatabaseName = args[i + 1];
                            i++;
                            break;

                        case "STATUS":
                            result = Action.Status;
                            break;

                        case "REQUESTID":
                        case "R":
                            this.requestId = args[i + 1];
                            i++;
                            break;

                        case "IMPORT":
                        case "I":
                            result = Action.Import;
                            break;

                        case "SELECTIVEEXPORTFILENAME":
                        case "SXF":
                            selectiveExportFileName = args[i + 1];
                            i++;
                            break;

                        case "EDITION":
                            string edstring = args[i + 1];
                            i++;
                            switch (edstring.ToUpper())
                            {
                                case "BUSINESS":
                                    this.azureEdition = edstring;
                                    break;
                                case "WEB":
                                    this.azureEdition = edstring;
                                    break;
                                default:
                                    throw new ArgumentException("Invalid argument for SQL Azure Edition.  (BUSINESS or WEB)");
                            }
                            break;

                        case "SIZE":
                            int size = int.Parse(args[i + 1]);
                            if (size > 0)
                            {
                                this.azureSize = size;
                            }
                            i++;
                            break;

#if DEBUG
                        case "ENDPOINT":
                            this.endpoint = args[i + 1];
                            i++;
                            break;

                        case "DEBUG":
                            Debugger.Launch();
                            break;
#endif

                        case "HELP":
                        case "H":
                        case "?":
                            result = Action.Help;
                            break;

                        default:
                            Console.WriteLine("[ERROR] UNKNOWN COMMAND: {0}", command);
                            result = Action.Help;
                            break;
                    }
                }
                else
                {
                    result = Action.Help;
                    break;
                }
            }

            ArgumentsValidater(result);

            Console.WriteLine("Arguments validation completed.");

            return result;
        }

        #region Arguments Validation
        /// <summary>
        /// Go through our arguments and ensure that we have valid parameters for the given action
        /// </summary>
        /// <param name="action"></param>
        internal void ArgumentsValidater(Action action)
        {
            if (action == Action.Import || action == Action.Export || action == Action.MaskedExport)
            {
                if (string.IsNullOrEmpty(this.connectionInfo.ServerName))
                {
                    throw new ArgumentException("Server name must be specified.");
                }

                if (string.IsNullOrEmpty(this.connectionInfo.DatabaseName))
                {
                    throw new ArgumentException("Database name must be specified.");
                }
            }

            // Verify the blob storage information
            if (action == Action.Import || action == Action.Export || action == Action.MaskedExport)
            {
                if (string.IsNullOrEmpty(this.blobUrl))
                {
                    throw new ArgumentException("BLOB Url must be specified.");
                }

                if (string.IsNullOrEmpty(this.blobAccessKey))
                {
                    throw new ArgumentException("BLOB Access Key must be specified.");
                }

                if (string.IsNullOrEmpty(this.blobAccessKeyType))
                {
                    throw new ArgumentException("Access Key Type must be specified as shared or storage level access.");
                }
            }

            if (action == Action.SelectiveExport)
            {
                if (selectiveExportFileName == null)
                {
                    throw new ArgumentException("Selective Export Input File containing the table names must be specified.");
                }
            }

            if (action == Action.Status)
            {
                if (string.IsNullOrEmpty(this.connectionInfo.ServerName))
                {
                    throw new ArgumentException("Server name must be specified.");
                }

                if (string.IsNullOrEmpty(this.connectionInfo.UserName))
                {
                    throw new ArgumentException("User Name must be specified.");
                }

                if (string.IsNullOrEmpty(this.connectionInfo.Password))
                {
                    throw new ArgumentException("User password must be specified.");
                }
            }

            // We need to figure out the datacenter before we can declare this complete
            if (action != Action.Invalid && action != Action.Help)
            {
                if (this.endpoint == null)
                {
                    this.dataCenter = DataCenterMapper.GetDatacenterFromServerName(this.connectionInfo.ServerName);
                }
                else
                {
                    this.dataCenter = new DataCenter();
                    this.dataCenter.Name = "User Provided";
                    this.dataCenter.Endpoint = this.endpoint;
                    this.dataCenter.SecurityMode = this.endpoint.StartsWith("http:", StringComparison.OrdinalIgnoreCase) ? WebHttpSecurityMode.None : WebHttpSecurityMode.Transport;
                }

                Console.WriteLine("Server has been mapped to {0} datacenter.", this.dataCenter.Name);
            }

            if (!string.IsNullOrEmpty(this.selectiveExportFileName))
            {
                if (action == Action.SelectiveExport && !System.IO.File.Exists(this.selectiveExportFileName))
                {
                    throw new ArgumentException(string.Format("Filename {0} does not appear to exist.", this.selectiveExportFileName));
                }
            }

        }
        #endregion
    }
}
