﻿using System;
using System.Reflection;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Threading;
using DacIEServiceCli.IEService;

namespace DacIEServiceCli
{
    public partial class Program
    {
        /// <summary>
        /// This enumeration indicates the actions that can be initiated by the 
        /// command line.
        /// </summary>
        internal enum Action
        {
            /// <summary>
            /// The action is invalid.
            /// </summary>
            Invalid,

            /// <summary>
            /// The requested action is to show help for the tool.
            /// </summary>
            Help,

            /// <summary>
            /// Submit an export operation
            /// </summary>
            Export,

            /// <summary>
            /// Submit an import operation
            /// </summary>
            Import,

            /// <summary>
            /// Submit a request for a status report
            /// </summary>
            Status,

            /// <summary>
            /// Perform a selective export operation on a bacpac
            /// </summary>
            SelectiveExport,

            /// <summary>
            /// Obfuscate data and perform an export of the maksed data onto a bacpac.
            /// </summary>
            MaskedExport,
        }

        private string azureEdition = string.Empty;
        private int azureSize = -1;
        private readonly ConnectionInfo connectionInfo = new ConnectionInfo();
        private string requestId = null;
        private DataCenter dataCenter = null;
        private string endpoint = null;
        private string selectiveExportFileName;

        private string blobUrl;
        private string blobAccessKey;
        private string blobAccessKeyType;
        public static readonly string StorageAccessKey = "Storage Access Key";
        public static readonly string SharedAccessKey = "Shared Access Key";

        static void Main(string[] args)
        {
            Program p = new Program();
            p.RunProgram(args);
        }

        /// <summary>
        /// This initializes the process, parses the command line, and handles exceptions.
        /// </summary>
        /// <param name="args">The command line arguments.</param>
        internal void RunProgram(string[] args)
        {
            Thread.GetDomain().UnhandledException += new UnhandledExceptionEventHandler(this.Program_UnhandledException);

            Program p = new Program();
            p.ShowLogo();

            try
            {
                switch (p.ProcessCommandLine(args))
                {
                    case Action.Export:
                        p.ExportAction();
                        break;
                    case Action.Import:
                        p.ImportAction();
                        break;
                    case Action.Status:
                        p.StatusAction();
                        break;
                    case Action.SelectiveExport:
                        p.SelectiveExportAction();
                        break;
                    case Action.MaskedExport:
                        p.MaskedExportAction();
                        break;

                    case Action.Help:
                    default:
                        p.Usage();
                        break;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("An error has occurred:");
                Console.WriteLine(e.Message);
                Exception inner = e.InnerException;
                while (inner != null)
                {
                    Console.WriteLine(inner.Message);
                    inner = inner.InnerException;
                    Console.WriteLine(string.Empty);
                }
                Console.WriteLine(string.Empty);
            }

            //Console.ReadLine();
        }


        /// <summary>
        /// This method shows copyright and version information for the tool in the console window.
        /// </summary>
        public void ShowLogo()
        {
            Assembly assembly = Assembly.GetExecutingAssembly();

            Console.WriteLine(Properties.Resources.Logo1, assembly.GetName().Version.ToString());
            Console.WriteLine("Copyright (C) Microsoft Corporation. All rights reserved.");
            Console.WriteLine("");
        }

        /// <summary>
        /// This method writes out help information on how to use the tool to the console window.
        /// </summary>
        public void Usage()
        {
            Console.WriteLine("Command Line Parameters:");
            Console.WriteLine("-H[elp] | -?                                  Show this help text");
            Console.WriteLine("-X[export]                                    Perform an export action(*4)");
            Console.WriteLine("-SX[SelectiveExport]                          Perform a selective export action.");
            Console.WriteLine("-MX[MaskedExport]                             Perform a masked export action.");
            Console.WriteLine("-I[mport]                                     Perform an import action");
            Console.WriteLine("-D[atabase] <database>                        Target database name");
            Console.WriteLine("-S[erver] <servername>                        SQL Azure Server Name");
            Console.WriteLine("-U[ser]                                       Database user name");
            Console.WriteLine("-P[assword]                                   Database user password");
            Console.WriteLine("-EDITION <business|web>                       SQL Azure edition (*2)");
            Console.WriteLine("-SIZE <1>                                     Max SQL Azure database size in GB(*2)");
            Console.WriteLine("-BLOB[URL] <url>                              Azure Blob URL (*6)");
            Console.WriteLine("-BLOBACCESS[K]EY <key>                        Azure Blob Access Key (*6)");
            Console.WriteLine("-ACCESSKEY[T]YPE <shared|storage>             Blob Access Key Type (*6)");
            Console.WriteLine("-STATUS                                       Status of operations (*5)");
            Console.WriteLine("-R[equestId] <requestId>                      Request Id of an operation, used for status (*5)");
            Console.WriteLine("-SXF[SELECTIVEEXPORTFILENAME] <filename>      Name of the file that contains the tables names in xml format for selective export. (*7)");
#if DEBUG
            Console.WriteLine("-ENDPOINT <endpoint>               Service endpoint");
            Console.WriteLine("-DEBUG                             Start with debugger attached");
#endif

            Console.WriteLine(string.Empty);
            Console.WriteLine(string.Empty);
            Console.WriteLine("Usage:");
            Console.WriteLine("Export a database to a bacpac in Azure Blob Storage:");
            Console.WriteLine("   {0} -S myserver  -U azureuser -P azurepwd -D northwind -X -BLOBACCESSKEY sharedkey -BLOBURL http://myurl/container/newfile.bacpac -ACCESSKEYTYPE shared", this.AppName);
            Console.WriteLine(string.Empty);

            Console.WriteLine("Selective Export a database to a bacpac in Azure Blob Storage:");
            Console.WriteLine("   {0} -S myserver  -U azureuser -P azurepwd -D northwind -SX -SXF selectiveExportInputFile.xml -BLOBACCESSKEY storageKey -BLOBURL http://myurl/container/newfile.bacpac -ACCESSKEYTYPE storage", this.AppName);
            Console.WriteLine(string.Empty);

            Console.WriteLine("Export a database with Masked data to a bacpac in Azure Blob Storage:");
            Console.WriteLine("   {0} -S myserver  -U azureuser -P azurepwd -D northwind -MX -BLOBACCESSKEY sharedkey -BLOBURL http://myurl/container/newfile.bacpac -ACCESSKEYTYPE storage", this.AppName);
            Console.WriteLine(string.Empty);

            Console.WriteLine("Import a bacpac to SQL Azure from Azure Blob Storage:");
            Console.WriteLine("   {0} -S myazure -U azureuser -P azurepwd -D nw_restored -I -EDITION web -SIZE 5 -BLOBACCESSKEY key -BLOBURL http://myurl/container/file.bacpac -ACCESSKEYTYPE storage", this.AppName);
            Console.WriteLine(string.Empty);

            Console.WriteLine("Find the status of the requests (requires server and a username / password) :");
            Console.WriteLine("   {0} -STATUS -S myazure -U myuser -P p@ssword [-R myreqId]", this.AppName);
            Console.WriteLine(string.Empty);

            Console.WriteLine("*Notes and caveats:");
            Console.WriteLine("1. On import the database must not exist.  A new database is");
            Console.WriteLine("   always created. SQL Azure uses system edition defaults if not set.");
            Console.WriteLine("2. Choose the SQL Azure options desired, this may impact billing.");
            Console.WriteLine("   (Only valid against SQL Azure)");
            Console.WriteLine("3. See SQL Server Books Online Topic: ");
            Console.WriteLine("   Understanding Data-tier Applications ");
            Console.WriteLine("   for more information about DAC.");
            Console.WriteLine("4. Blob will be overwritten if it exists.");
            Console.WriteLine("5. Status is for all recent submissions on a server level.");
            Console.WriteLine("   A valid username and password for this server must be supplied.");
            Console.WriteLine("   To see status for a single requestid it must be provided.");
            Console.WriteLine("6. The Azure Storage may be used in storage or shared access modes.");
            Console.WriteLine("   To generate a shared access key see the FAQ.");
            Console.WriteLine("7. The input file needed for selective export has a specific XML schema.");
            Console.WriteLine("   Please check the download site for a sample.");

            Console.WriteLine(string.Empty);
        }

        internal string AppName
        {
            get
            {
                Assembly assembly = Assembly.GetEntryAssembly();

                // If called from unmanaged code (like MSTEST) the GetEntryAssembly will return null
                if (assembly == null)
                {
                    assembly = Assembly.GetExecutingAssembly();
                }

                if (assembly != null)
                {
                    return assembly.GetName().Name;
                }

                return string.Empty;
            }
        }

        private static BlobCredentials GetBlobCredentials(string blobUrl, string accessKey, string accessKeyType)
        {
            if (Program.StorageAccessKey.Equals(accessKeyType))
            {
                BlobStorageAccessKeyCredentials storageCredentials = new BlobStorageAccessKeyCredentials();
                storageCredentials.StorageAccessKey = accessKey;
                storageCredentials.Uri = blobUrl;
                return storageCredentials;
            }
            else if (Program.SharedAccessKey.Equals(accessKeyType))
            {
                BlobSharedAccessKeyCredentials sharedCredentials = new BlobSharedAccessKeyCredentials();
                sharedCredentials.SharedAccessKey = accessKey;
                sharedCredentials.Uri = blobUrl;
                return sharedCredentials;
            }

            return null;
        }

        private IDACWebService CreateWebServiceChannel()
        {
            var factory = new ChannelFactory<IDACWebService>(new WebHttpBinding(this.dataCenter.SecurityMode), new System.ServiceModel.EndpointAddress(this.dataCenter.Endpoint));
            factory.Endpoint.Behaviors.Add(new WebHttpBehavior());
            return factory.CreateChannel();
        }
    }
}
